c
      subroutine axes (xx, yy, xlen, x1, x2, ixy)
c
c neat subroutine to plot linear axes -- r.l. parker & l. shure
c                                     -- modified by k. mackenzie
c
c$$$$ calls plot, symbol, justfy, nicer
c
c  plots either an x- or a y-axis beginning at position (xx,yy)
c   (in inches) of length  xlen (inches) with tick
c  marks at reasonable places between the limits  x1, x2.
c  x1 may be greater or less than x2 (allowing reversed axis).
c  the axis may be set to be any of the four sides of a plot:
c  --- ticks labeled with numbers
c  ixy =1 means x-axis (for the bottom), ixy =-1 means x-axis (for the top)
c  ixy =2 means y-axis (for the left),   ixy =-2 means y-axis (for the right).
c  --- ticks not labeled
c  ixy =3 means x-axis (for the bottom), ixy =-4 means x-axis (for the top)
c  ixy =4 means y-axis (for the left),   ixy =-4 means y-axis (for the right).
c
      character*40 clabel, ifmt*10
      real*4 xa(2), s(4)
      integer*4 label(10)
      logical qnum
      common /ilogik/ logik, xslg
      common /backs/ backy, ht
      data ticfac /1.6/, ht /0.15/, logik /0/
      save
c
      iten(x)= int(500.001 + alog10(x)) - 500
c
      qnum= .true.
      htix= .1
      iyx= ixy
      if (iyx .lt. 0) then
         iyx= iabs (iyx)
         htix= -.1
      end if
      if (iyx .gt. 2) then
         qnum= .false.
         iyx= iyx - 2
      end if
      xa(1)= x1
      xa(2)= ticfac * amin1 (1.0, 10.0/xlen) * (x2-x1) + x1
      call nicer (xa, xa, xt)
c  set up format for numerical annotation on the axis.
      nsiz= iten ( amax1 (abs (x2), abs(x1)) )
      ntix= iten (abs (xt))
      nfld= 3 + max0 (nsiz, -ntix, nsiz-ntix)
      if (nfld .gt. 7) go to 1000
c  width of field less than 8 characters - use an f-format.
      ndp= max0 (0, -ntix)
      write (ifmt(1:8), 100) nfld, ndp
100   format ('(f',i1,'.',i1,')  ')
      if (ndp .eq. 0) nfld= nfld - 1
      go to 1100
c  width of field more than 7 characters - use a g format.
 1000 ndp= nsiz - ntix + 1
      nfld= 7 + ndp
      write (ifmt(1:10), 110) nfld, ndp
110   format ('(1pg', i2, '.', i2, ')' )
c
 1100 call plot (xx, yy, 3)
      xs= xlen / (x2-x1)
      n1= xa(1) / xt
      n2= x2/xt  +  1.0
      go to (2100, 3100), iyx
c
c  draw the x axis.
 2100 kskip= 1.3  +  ht * nfld * (logik+1) / (xs*xt)
      xlab= -1.5 * ht  +  yy
      if (htix .lt. 0.) xlab= .5*ht  +  yy
      htic= htix + yy
      do 2500 n= n1, n2
      x= n * xt
      xinch= xs * (x-x1)
      if (logik .eq. 1) xinch= xslg * alog10 (x/x1)
c  plot next section of axis and the tick on the right.
      if (xinch .lt. -0.01 .or. xinch .gt. xlen+0.01) go to 2500
      xinch= xinch + xx
      call plot (xinch, yy,  2)
      call plot (xinch, htic, 2)
c  write numerical annotation.
      if (mod (n, kskip) .ne. 0) go to 2400
      write (clabel, ifmt) x
      read (clabel(1:nfld), '(10a4)') label
      call justfy (s, ht, label, nfld)
      if (qnum) call symbol (xinch-s(2), xlab, ht, label, 0.0, nfld)
c  move back onto axis with pen up.
 2400 call plot (xinch, yy, 3)
 2500 continue
c  plot the last little piece of the axis and thicken the axis.
      xxx= xx + xlen
      call plot (xxx, yy, 2)
      yyy= yy + sign (.008, htix)
      call plot (xxx, yyy, 2)
      call plot (xx, yyy, 2)
      call plot (xx, yy, 3)
c  return to calling program
      return
c
c  draw the y axis.
 3100 half= -0.5 * ht
      signeg= ht
      if (abs (x1) .gt. abs (x2) .and. x1 .lt. 0.) signeg= 0.
      if (abs (x2) .gt. abs (x1) .and. x2 .lt. 0.) signeg= 0.
      backy= half
      kskip= 1.5  +  ht * (logik+1) / (xs*xt)
      htic= htix + xx
      do 3500 n= n1, n2
      y= n * xt
      yinch= xs * (y-x1)
      if (logik .eq. 1) yinch= xslg * alog10 (y / x1)
c  plot next section of axis and the tick on the right.
      if (yinch .lt. -0.01 .or. yinch .gt. xlen+0.01) go to 3500
      yinch= yinch + yy
      call plot (xx, yinch, 2)
      call plot (htic, yinch, 2)
c  write numerical annotation.
      if (mod (n, kskip) .ne. 0) go to 3400
      write (clabel, ifmt) y
      read (clabel(1:nfld), '(10a4)') label
      call justfy (s, ht, label, nfld)
      backy= amax1 (backy, s(3)-s(1)-half)
      ylab= half - s(3)
      if (htix .lt. 0.) ylab= .5*ht - signeg
      if (qnum) then
         call symbol (xx+ylab, yinch+half, ht, label, 0.0, nfld)
      end if
 3400 call plot (xx, yinch, 3)
 3500 continue
c  plot last little piece of axis and thicken axis.
      yyy= yy + xlen
      call plot (xx, yyy, 2)
      xxx= xx + sign (.008, htix)
      call plot (xxx, yyy, 2)
      call plot (xxx, yy,  2)
      call plot (xx, yy, 3)
c  return to calling program
      return
      end
