      subroutine chrcod(text,ntext)
c  given text string in text, ntext characters
c  returns ichr containing nchr symbol numbers or codes for
c  space (1000), begin superscripting (1001), begin
c  subscripting (1002), end super/sub-scripting (1003)
c  backspace (1004), vector (1005), or hat (1006)
c  change of font commands are decoded and executed internally
c
      common /offset/ ioff,just1,just2
      common /ajust/nchr,ichr(350)
      character*(*) text
      integer irlu(95),iilu(95),iglu(26)
      save /offset/,/ajust/
c     data ioff/0/
c  irlu is a look-up table for roman characters arranged by
c  integer value for the ascii character set with an
c  offset to remove the 31 nonprinting control characters.
c  irlu returns with the symbol number or, if no symbol
c  exists, the code for space.
      data irlu/1000,416,428,411,72,418,419,432,67,68,69,63,70,
     $          64,71,65,53,54,55,56,57,58,59,60,61,62,414,415,
     $          385,66,386,417,407,1,2,3,4,5,6,7,8,9,10,11,12,13,
     $          14,15,16,17,18,19,20,21,22,23,24,25,26,409,1000,
     $          410,408,1000,1000,27,28,29,30,31,32,33,34,35,36,
     $          37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,
     $          405,427,406,424/
c  iilu is a look-up table for italic characters only.  it is
c  identical to irlu with four italic special symbols substituted
c  for regular ones.
      data iilu/1000,422,1000,411,72,418,419,1000,67,68,69,63,70,
     $          64,71,65,53,54,55,56,57,58,59,60,61,62,420,421,
     $          385,66,386,423,407,1,2,3,4,5,6,7,8,9,10,11,12,13,
     $          14,15,16,17,18,19,20,21,22,23,24,25,26,409,1000,
     $          410,1000,1000,1000,27,28,29,30,31,32,33,34,35,36,
     $          37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,
     $          405,427,406,424/
c  iglu is a look-up table for greek characters arranged by the
c  integer value of their roman expression with a=1, b=2, etc.
c  ambiguous cases give 25 for epsilon or eta, 26 for omega or
c  omicron, 27 for phi,pi,or psi, and 28 for tau or theta.  additional
c  letters must be checked for these case.  a value of 50 is returned
c  for those roman letters which have no corresponding greek letter.
      data iglu/1,2,22,4,25,50,3,50,9,50,10,11,12,13,26,27,50,17,18,
     $             28,20,50,50,14,50,6/
c finds length of string with blanks trimmed from right end.
c Set ioff = 0  (maybe)
      do 10 n=ntext,1,-1
 10     if(text(n:n).ne.' ')goto 15
      nchr=0
      return
 15   nt=n
c  scan text character by character
      k=1
      j=1
c  k is current address of character in text
c  j is index of next symbol code in ichr
   20 if(k.gt.n)then
        nchr=j-1
        return
      endif
      if(text(k:k).ne.'\\')then
c  roman character or keyboard symbol
        if(text(k:k).eq.'}')then
c  check for closing curly bracket-if found, return 1003
          ichr(j)=1003
          j=j+1
          k=k+1
          goto 20
        endif
c  ichar returns integer ascii value of character
c  offset by nonprinting characters to get entry in look-up table
        ic=ichar(text(k:k))-ichar(' ')+1
        if(ic.le.0)then
c  nonprinting control character-error return
          ichr(j)=1000
        elseif (ioff.ne.240)then
c  not italic font
          ichr(j)=irlu(ic)
        else
c  italic font
          ichr(j)=iilu(ic)
        endif
c  add offset for font if not a special symbol
        if(ichr(j).lt.385)ichr(j)=ichr(j)+ioff
          j=j+1
          k=k+1
          goto 20
        else
c  backslash found
c  check next four characters for four digit number
          k=k+1
        read(text(k:k+3),25,err=50)number
   25 format(i4)
c  number found-check its validity
          ic=number-1000
          if((ic.gt.0).and.(ic.lt.433))then
c  valid symbol code
            ichr(j)=ic
          elseif ((ic.gt.999).and.(ic.lt.1004))then
c  valid command code
            ichr(j)=ic
          else
c not recognized-error return
            ichr(j)=1000
          endif
          j=j+1
c  move beyond closing backslash-ignore extra characters
c  function index returns offset of second substring in first
c  returns 0 if substring not found
           l=index(text(k:nt),'\\')
           if(l.eq.0)then
             k=nt+1
           else
             k=k+l
           endif
           goto 20
   50      continue
c  not a number
c  check for font change command
         if(text(k:k+2).eq.'sim'.or.text(k:k+2).eq.'SIM')then
c  simplex font
           ioff=0
         elseif(text(k:k+1).eq.'co'.or.text(k:k+1).eq.'CO')then
c  complex font
           ioff=120
         elseif(text(k:k+1).eq.'it'.or.text(k:k+1).eq.'IT')then
c  italic font
           ioff=240
         elseif (text(k:k+1).eq.'du'.or.text(k:k+1).eq.'DU')then
c  duplex font
           ioff=312
c  found the back-space code
         elseif(text(k:k+1).eq.'bs'.or.text(k:k+1).eq.'BS') then
           ichr(j)=1004
           j=j+1
           k=k+3
           go to 20
c  check for super/sub-script command
         elseif(text(k:k+3).eq.'sup{'.or.text(k:k+3).eq.'SUP{')then
c  begin superscripting
           ichr(j)=1001
           j=j+1
           k=k+4
           goto 20
         elseif (text(k:k+3).eq.'sub{'.or.text(k:k+3).eq.'SUB{')then
c  begin subscripting
           ichr(j)=1002
           j=j+1
           k=k+4
           goto 20
         else
c  greek character or invalid character
           ic=ichar(text(k:k))
           igoff=min0(ioff, 120)
           if(ioff.eq.312)igoff=0
           if((ic.ge.ichar('A')).and.(ic.le.ichar('Z')))then
c  upper case
             igr=72
             ico=ichar('A')-1
           elseif((ic.ge.ichar('a')).and.(ic.le.ichar('z')))then
c  lower case
             igr=96
             ico=ichar('a')-1
           else
c  not a letter-error return
             ichr(j)=1000
             j=j+1
             l=index(text(k:nt),'\\')
             if(l.eq.0)then
               k=nt+1
             else
               k=k+l
             endif
             goto 20
           endif
c  look up the character
           ig=iglu(ic-ico)
           if(ig.lt.25)then
c  unambiguous greek letter
             ichr(j)=ig+igr+igoff
           elseif (ig.eq.25)then
c  epsilon or eta
             ib=ichar(text(k+1:k+1))-ico
             if(ib.eq.16)then
c  epsilon
               ichr(j)=5+igr+igoff
             elseif (ib.eq.20)then
c  eta
               ichr(j)=7+igr+igoff
             else
c  not a greek character--error return
               ichr(j)=1000
             endif
         elseif (ig.eq.26)then
c  omega or omicron
           ib=ichar(text(k+1:k+1))-ico
           if(ib.ne.13)then
c not a greek character-error return
             ichr(j)=1000
           else
             ic=ichar(text(k+2:k+2))-ico
             if(ic.eq.5)then
c  omega
               ichr(j)=24+igr+igoff
             elseif (ic.eq.9)then
c  omicron
               ichr(j)=15+igr+igoff
             else
c  not a greek character-error return
               ichr(j)=1000
             endif
           endif
         elseif (ig.eq.27)then
c  phi,pi, or psi
           ib=ichar(text(k+1:k+1))-ico
           if(ib.eq.8)then
c  phi
             ichr(j)=21+igr+igoff
           elseif (ib.eq.9)then
c  pi
             ichr(j)=16+igr+igoff
           elseif (ib.eq.19)then
c  psi
             ichr(j)=23+igr+igoff
           else
c  not a greek character-error return
             ichr(j)=1000
           endif
           elseif (ig.eq.28)then
c tau or theta
             ib=ichar(text(k+1:k+1))-ico
             if(ib.eq.1)then
c  tau
               ichr(j)=19+igr+igoff
             elseif(ib.eq.8)then
c  theta
               ichr(j)=8+igr+igoff
             else
c  not a greek character-error return
               ichr(j)=1000
             endif
           else
c  not a greek character-error return
             ichr(j)=1000
           endif
          j=j+1
        endif
        l=index(text(k:nt),'\\')
        if(l.eq.0)then
          k=nt+1
        else
          k=k+l
        endif
        goto 20
      endif
      end
