c ******** plot3d **********
      subroutine plot3d(ivxyz, xdata, ydata, zdata, xscale,
     * yscale, zscale, nline, npnts, phi, theta, xref,
     * yref, xlenth, mask, vertex)
c
c   copyright 1974, association for computing machinery, inc.
c
c  steven l. watkins, algorithm 483, masked three dimensional plot with
c  rotations, communications of the acm, sept. 1974.  reprinting
c  privileges were granted by permission of the association for
c  computing machinery.
c
c masked 3-dimensional plot program with rotations
c this routine will accept 3-dimensional data in various
c forms as input, rotate it in 3-space to any angle
c and plot the projection of the resulting figure onto the
c x-y plane.  linear interpolation is used between data
c points.  those lines of a figure which should be hidden by
c a previous line are masked.  the x-axis is parallel to the paper
c while the y-axis runs across the paper.  the z-axis comes out
c of the paper.
c the masking technique used by this routine is based on
c two premises:
c lines in the foreground (positive z direction)
c are plotted before lines in the background.
c a line or portion of a line is masked (hidden) if
c it lies within the region bounded by previously
c plotted lines.
c each call to plot3d causes one line of a figure to be
c plotted.
c two parameters of the plotter are set on the initial call
c for each figure -
c (pipi) is the number of plotter increments per inch.
c (ylenth) is the width of the paper (y-direction).
c input parameters -
c (ivxyz) is a four digit decimal integer which is used to
c select various input/output options.  these digits, in
c decreasing order of magnitude, will be referred to as v,
c x, y, and z.
c if v .ne. 0, the vertices of the current figure and their
c projection onto the y=0 plane, will be stored in a 16
c entry real array (vertex), and will be updated as each
c line is plotted.  these coordinates are in inches and are
c relative to the current plotter origin.  the x y pairs
c are ordered so that the first pair corresponds to the
c first point of the figure, the second pair corresponds
c to the last point of the first line, and the following
c pairs are ordered in a circular fashion.  the pairs on the
c y=0 plane of the figure, then follow in the same order.
c if v=0, the vertex parameter is ignored, but should not
c be deleted
c if x=0, the x-components of this line are assumed to be
c equally spaced, and are computed by
c x(i)=xdata+(i-1)*xscale
c where (xdata) is the initial value in inches and (xscale)
c is the spacing between points in inches.  if x .ne. 0, the
c x-components of this line are read from an array and
c modified by
c x(i)=xdata(i)*xscale
c where (xscale) is a scale factor.
c the same relations hold for the y-components, that is, if
c y=0
c y(i)=ydata+(i-1)*yscale
c and if y .ne. 0
c y(i)=ydata(i)*yscale
c if z=0, the z-components of this line are all assumed to
c be equal, and are computed by
c z(i)=zdata+(nline-1)*zscale
c where (nline) is some integer associated with this line.
c if z .ne. 0, again we have
c z(i)=zdata(i)*zscale
c when (nline) is equal to one, it indicates the beginning
c of a new figure.  a call to plot3d with (nline) equal to
c zero before initiating a new figure simulates a line drawn
c at the bottom of the page.  therefore only those portions
c of a line lying above all previous lines will be plotted.
c all other parameters are ignored on such a call.
c (npnts) is the number of points on this line, and may be
c altered from line to line.
c (phi) and (theta) are the two angles (in degrees) used to
c specify the desired 3-dimensional rotation.  the following
c two definitions of these rotations are equivalent -
c in terms of rotations of axes, the initial system of axes,
c xyz, is rotated by an angle (phi) counterclockwise about
c the y-axis, and the resultant system is labeled the tuv
c axes.  the tuv axes are then rotated by an angle (theta)
c counterclockwise about the t-axis, and this final system
c is labeled the pqr axes.  the plotted figure is the
c projection of the original figure onto the pq-plane.
c in terms of rotations of coordinates, the figure is first
c rotated by an angle (theta) clockwise about the x-axis.
c the resultant figure is then rotated by an angle (phi)
c clockwise about its y-axis.  the plotted figure is the
c projection of this final figure onto the xy-plane.
c warning: some rotations will alter the foreground/
c background relationships between the lines, and
c thus the order in which they should be plotted.
c (xref) and (yref) are the coordinates, in inches,
c relative to the plotter origin, to be used as the origin
c of the figure.
c (xlenth) is the length, in inches, to which the plot is
c restricted.  any point which exceeds this limit, or the
c limits of the paper in the y direction (ylenth), will be
c set to that limit.
c (mask) is an integer array of 2*xlenth*pipi entries which
c is used to store the mask.  the contents of this array
c should not be altered during the ploting of any given
c figure.
c all parameters except (mask) and (vertex) are returned
c unchanged.
c between any two calls for the same figure, any parameter
c can be meaningfully changed except (xlenth), (mask), and
c (vertex).
      common /pipi/ pipi, ylenth
      save /pipi/
      save nypi
      save inci, indv, indx, indy, indz, a11, a13, a21, a22, a23
      save limitx, i
      integer*4 high, oldhi, oldlow
      real*4 xdata(1), ydata(1), zdata(1), mask(1), vertex(1)
      data init, jvxyz, sphi, stheta /-1, -1, -1.0e37, -1.0e37/
c set plotter parameters
      data pipi, ylenth /100., 10.9/
c pipi may be set to a value less than the actual number of plotter
c increments per inch to conserve space or time, the resolution of the
c plot will be reduced accordingly.
c ylenth is the paper width available for plotting, 10.9 for a 12 inch
c plotter, 29 for a 30 inch plotter.
c pipi and ylenth may be changed by including
c common / pipi / pipi,ylenth  in the calling program and setting
c the variables to the desired values.
c
c *** modifications ***
c   9/21/84 krm -- converted to fortran77
c               -- converted 'iplot' calls to standard calcomp 'plot' while
c                    maintaining 'pipi' scaling
c               -- converted 'iround' calls to fortran77 'nint'
c               -- at initialization, now resets origin to current position
c
c *** end modifications ***
c
c initialization procedures
c initialization procedure for a new figure
      rpipi= 1./pipi
c test for special mask modifying call
      if (nline.eq.0) go to 550
c determine if initialization is required
      if (nline.ne.1) go to 20
c set plotter parameters
      nypi = ylenth*pipi
c reset plotter origin to current position
      i = nypi + 100
c      call plot(0., 0., -3)
c compute length of plot page in increments
      limitx = xlenth*pipi + 0.5
      i = limitx + limitx
c initialize masking array over the length of the plot page
      do 10 k=1,i
      mask(k) = init
   10 continue
      init = -1
c set the necessary indicators for the first line of a new
c figure
      inci = -1
      i = 0
c input type and vertex initialization
c determine if initialization is required
   20 if (jvxyz.eq.ivxyz) go to 70
c set indicators for types of input data and saving vertices
      jvxyz = ivxyz
      indz = 1
      indy = 1
      indx = 1
      indv = 1
      if (jvxyz.lt.1000) go to 30
      indv = 2
      jvxyz = jvxyz - 1000
   30 if (jvxyz.lt.100) go to 40
      indx = 2
      jvxyz = jvxyz -100
   40 if (jvxyz.lt.10) go to 50
      indy = 2
      jvxyz = jvxyz - 10
   50 if (jvxyz.lt.1) go to 60
      indz = 2
   60 jvxyz = ivxyz
c rotation initialization
c determine if initialization is required
   70 if (phi.eq.sphi .and. theta.eq.stheta) go to 80
c compute rotation factors
      sphi = sin (0.0174532925*phi)
      cphi = cos (0.0174532925*phi)
      stheta = sin (0.0174532925*theta)
      ctheta = cos (0.0174532925*theta)
      a11 = cphi
      a13 = -sphi
      a21 = stheta*sphi
      a22 = ctheta
      a23 = stheta*cphi
      sphi = phi
      stheta = theta
c processing procedures
c set flag to move through the data arrays in the opposite
c direction
   80 inci = -inci
c set indicator to the first point to be processed
      if (i.ne.0) i = npnts + 1
c loop to process each point in the data arrays
      do 530 k=1,npnts
c data calculation
      i = i + inci
      go to (90,100), indx
   90 x = xdata(1) + (i-1)*xscale
      go to 110
  100 x = xdata(i)*xscale
  110 go to (120,130), indy
  120 y = ydata(1) + (i-1)*yscale
      go to 140
  130 y = ydata(i)*yscale
  140 go to (150,160), indz
  150 z = zdata(1) + (nline-1)*zscale
      go to 170
  160 z = zdata(i)*zscale
c data rotation
  170 xxx = a11*x + a13*z + xref
      xx = xxx
      ix = nint(xx*pipi)
      yyy = a21*x + a23*z + yref
      yy = yyy + a22*y
      iy = nint(yy*pipi)
c restrict figure to plot page
      if (ix.le.0) ix = 1
      if (ix.gt.limitx) ix = limitx
      if (iy.lt.10) iy = 10
      if (iy.gt.nypi) iy = nypi
      if (k.ne.1) go to 250
c (loc) is the position of the previous point with respect
c to the mask
c +1 above the mask
c 0 within the limits of the mask
c -1 below the mask
c procedure for initial point of each line
c locate initial point with respect to the mask then
c update the mask
      low = ix + ix
      high = low - 1
      mlow = mask(low)
      mhigh = mask(high)
      if (mhigh-iy) 200, 210, 180
  180 if (mlow-iy) 190, 230, 220
  190 locold = 0
      go to 240
  200 mask(high) = iy
      if (mlow.eq.-1) mask(low) = iy
  210 locold = +1
      go to 240
  220 mask(low) = iy
  230 locold = -1
c move the raised pen to this initial point
  240 call plot(ix*rpipi, iy*rpipi, 3)
      jx = ix
      jy = iy
      iyref = iy
c store vertices if requested
      if (indv.eq.1) go to 530
      index = inci + 6
      vertex(index) = xx
      vertex(index+1) = yy
      vertex(index+8) = xxx
      vertex(index+9) = yyy
      if (nline.ne.1) go to 530
      vertex(1) = xx
      vertex(2) = yy
      vertex(9) = xxx
      vertex(10) = yyy
      go to 530
c special case where change in x coordinate is zero
c a special provision is made at this point so that a line
c will not mask itself as long as the x coordinate remains
c constant
  250 if (ix.ne.jx) go to 260
      jy = iy
      go to 280
c compute constants for linear interpolation
  260 yinc = real(iy-jy)/abs(real(ix-jx))
      incx = (ix-jx)/iabs(ix-jx)
      yj = jy
c perform linear interpolation at each incremental step on
c the x axis
  270 jx = jx + incx
      yj = yj + yinc
      jy = nint(yj)
c locate the current point with respect to the mask at that
c point then plot the increment as a function of the
c location of the previous point with respect to its mask
      low = jx + jx
      high = low - 1
      mlow = mask(low)
      mhigh = mask(high)
  280 if (mhigh-jy) 300, 300, 290
  290 if (mlow-jy) 310, 320, 320
c the current point is above the mask
  300 loc = + 1
      if (locold) 360, 370, 430
c the current point is within the mask
  310 loc = 0
      if (locold) 340, 350, 330
c the current point is below the mask
  320 loc = -1
      if (locold) 510, 450, 440
c plot from above the mask to within the mask
  330 if (mhigh.le.iyref) call plot(jx*rpipi, mhigh*rpipi, 2)
      go to 350
c plot from below the mask to within the mask
  340 if (mlow.ge.iyref) call plot(jx*rpipi, mlow*rpipi, 2)
c plot from within the mask to within the mask
  350 call plot(jx*rpipi, jy*rpipi, 3)
      go to 520
c plot from below the mask to above the mask
  360 if (mlow-iyref) 370, 380, 380
c plot from within the mask to above the mask
  370 if (mhigh-iyref) 400, 390, 390
  380 call plot(jx*rpipi, mlow*rpipi, 2)
  390 call plot(jx*rpipi, mhigh*rpipi, 3)
      go to 430
  400 if (mhigh.eq.-1) go to 430
      oldhi = high - 2*incx
      if (mask(oldhi)-jy) 420, 420, 410
  410 call plot(jx*rpipi, jy*rpipi, 3)
      go to 430
  420 call plot((jx-incx)*rpipi, mask(oldhi)*rpipi, 3)
c plot from above the mask to above the mask
  430 mask(high) = jy
      if (mlow.eq.-1) mask(low) = jy
      call plot(jx*rpipi, jy*rpipi, 2)
      go to 520
c plot from above the mask to below the mask
  440 if (mhigh-iyref) 460, 460, 450
c plot from within the mask to below the mask
  450 if (mlow-iyref) 470, 470, 480
  460 call plot(jx*rpipi, mhigh*rpipi, 2)
  470 call plot(jx*rpipi, mlow*rpipi, 3)
      go to 510
  480 oldlow = low - 2*incx
      if (mask(oldlow)-jy) 490, 500, 500
  490 call plot(jx*rpipi, jy*rpipi, 3)
      go to 510
  500 call plot((jx-incx)*rpipi, mask(oldlow)*rpipi, 3)
c plot from below the mask to below the mask
  510 mask(low) = jy
      call plot(jx*rpipi, jy*rpipi, 2)
  520 iyref = jy
      locold = loc
      if (jx.ne.ix) go to 270
  530 continue
c raise pen
      call plot(jx*rpipi, jy*rpipi, 3)
c store vertices if requested
      if (indv.eq.1) go to 540
      index = -inci + 6
      vertex(index) = xx
      vertex(index+1) = yy
      vertex(index+8) = xxx
      vertex(index+9) = yyy
      if (nline.ne.1) go to 540
      vertex(3) = xx
      vertex(4) = yy
      vertex(11) = xxx
      vertex(12) = yyy
  540 i = i - 1
c return to calling program
      return
c option to modify the masking technique to be used on the
c following figure so as to plot only above all previous
c lines.
  550 init = 0
      return
      end
